/*
 * bbnet
 * BIG BROTHER NETWORK TESTER
 * REPLACEMENT FOR LYNX AND MAYBE TELNET TESTS
 * Sean MacGuire - The MacLawran Group Inc.
 *
 * Format: bbnet ARG
 *
 * ARG:		http://blah.com/bobo	HTTP TESTING
 *		blah.com		TELNET TESTING (OR ANY OTHER PORT)
 *
 * bbnet understands real machine names and port numbers
 * i.e. blah.com:21 will open a connection to the FTP server
 *
 * Displays whatever answers on that port/address to stdout
 *
 * Returns: 	0	OK
 *		1	Host not found
 *		2	Can't connect to server
 *		3	Timeout
 */

#include "bb.h"		/* THE BIG BROTHER INCLUDE FILE */
#include <netdb.h>	/* FOR HOST INFO */

int http=0;		/* ARE WE HTTP TESTING? */
unsigned long ip;	/* THE IP ADDRESS WE'RE TESTING */
int port;		/* THE PORT NUMBER WE'RE TESTING */
char *dir;		/* DIRECTORY */

jmp_buf env;  

main(argc, argv)
int argc;
char *argv[];
{
	int p;
	char *machine, *ptr;
	if (argc == 3) p=80;		/* HTTP TEST */
	else if (argc == 2) p=23;	/* TELNET */
	else {
		fprintf(stderr, "bbnet: incorrect number of arguments\n");
		fprintf(stderr, "Format: bbnet machine[:port]|http-request\n");
		exit(1);
	}
	/*
	 * PARSE THE ARGUMENT.  THIS SHOULD BE A DROP-IN REPLACEMENT
	 * FOR lynx, UGH.  THAT MEANS THE FIRST ARGUMENT MAY BE A REALLY
	 * NASTY THING.
	 */
	machine = (char *)malloc(MAXLINE);
	strcpy(machine, argv[1]);

	if (strncmp(machine, "http://", 7) == 0) {
		http = 1;
		debug("We have an http request\n");
		machine += 7;
		port = 80;	/* STANDARD HTTP PORT */
	}
	else port = 23;		/* TELNET DEFAULT */
	setup(machine);		/* DO SETUP */
	bnetsend();		/* DO TEST */
	exit(0);
}

/*
 * THIS FUNCTION SETS UP EVERYTHING WE NEED TO CALL THE PORT
 */
setup(box)
char *box;
{
	char *eob = 0, *ptr,*newdir;
	struct hostent *ho = 0;

	ptr = (char *)strchr(box, ':');
	if (ptr != (char *)NULL) {
		sscanf(ptr, ":%d", &port);
		eob = ptr;
	}
	debug("GOT PORT: %d\n", port);
	dir = (char *)strchr(box, '/');

	if (dir) {
		newdir = (char *)malloc(strlen(dir) + 1);
		strcpy(newdir, dir);
		debug("DIRECTORY = %s\n", newdir);
		*dir = '\0';
	}
	else debug("NO DIR\n");
	if (eob) *eob = '\0';		/* NOW GET MACHINE */

	debug("BOX: %s\n", box);

	if (ip = inet_addr(box) == -1) {	/* IS IT IN DOT FMT */
		if (*box == '\0') {
			debug("BOX IS LOCAL\n");
			ho = gethostbyname("localhost");
		}
		else ho = gethostbyname(box);

		if (ho != (struct hostent *)NULL) {
			ip = (unsigned long)ho->h_addr;
		}
		else {	/* TEST FOR AN IP-ADDRESS */
			fprintf(stderr, "Invalid host request: %s\n", box);
			exit(1);	/* NO HOST */
		}
	/*
	 * IF WE MAKE IT HERE, WE HAVE TO COPY THE HOSTNAME 
	 * INTO THE ADDRESS STRUCTURE
	 */
		memcpy(&ip, ho->h_addr, ho->h_length);
	}
	dir=newdir;			/* SET DIRECTORY */
}

bnetsend()
{
 	int n, sockfd, newsockfd, child, len, timeout();
        struct sockaddr_in serv_addr, cli_addr;
	char line[MAXLINE];

        debug("MAKING SOCKET CONNECTION...\n");
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "Can't open stream socket");
		exit(2);
        }
        debug("SOCKET CONNECTED OK...\n");

        bzero((char *) &serv_addr, sizeof(serv_addr));
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = ip;
        serv_addr.sin_port = htons(port);

	/*
	 * THIS IS A LITTLE NASTY
	 * IF THE SERVER'S NOT UP, THIS WILL WAIT A LONG TIME BEFORE
	 * IT TIMES OUT.  SO HANDLE THAT HERE... IT GETS 30 SECONDS TO ANSWER
	 */
 	if (setjmp(env) != 0) {
                debug("CALLING SETJMP\n");
        }
        signal(SIGALRM, (void *)timeout);
        alarm(30);

        /* OPEN SOCKET */
        if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
                fprintf(stderr, "\nbnet: CAN'T BIND TO SOCKET");
        }
        if (connect(sockfd, (struct sockaddr *) &serv_addr,
                        sizeof(serv_addr)) < 0) {
                fprintf(stderr, "\nbnet: CAN'T CONNECT TO SERVER");
        }
	/*
	 * HTTP REQUESTS ARE SPECIAL
	 */
	if (http) {
		if (dir) sprintf(line, "GET %s\n", dir);
		else sprintf(line, "GET /\n");
	}
        n = send(sockfd, line, strlen(line),0);
	line[0]='\0';
	n = recv(sockfd, line, MAXLINE, 0);
	puts(line);			/* DUMP OUT THE LINE */
	close(sockfd);
	return(n);
}

timeout()
{
	fprintf(stderr, "bbnet: CONNECTION TIMED OUT...");
	exit(3);
}
